﻿//-----------------------------------------------------------------------
// <copyright file="ListViewItemComparer.cs" company="MSCRMTools">
//     Copyright © MSCRMTools. All rights reserved.
// </copyright>
// <author>tanguy92</author>
// <date>21/02/2011</date>
// <codeplexurl>http://searchablepropertyupdater.codeplex.com</codeplexurl>
// <summary>Compares two listview items for sorting</summary>
//-----------------------------------------------------------------------
using System;
using System.Collections;
using System.Windows.Forms;

namespace SearchablePropertyUpdater2011
{
    /// <summary>
    /// Compares two listview items for sorting
    /// </summary>
    class ListViewItemComparer : IComparer
    {
        #region Variables

        /// <summary>
        /// Index of sorting column
        /// </summary>
        private int col;

        /// <summary>
        /// Sort order
        /// </summary>
        private SortOrder innerOrder;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of class ListViewItemComparer
        /// </summary>
        public ListViewItemComparer()
        {
            col = 0;
            innerOrder = SortOrder.Ascending;
        }

        /// <summary>
        /// Initializes a new instance of class ListViewItemComparer
        /// </summary>
        /// <param name="column">Index of sorting column</param>
        /// <param name="order">Sort order</param>
        public ListViewItemComparer(int column,SortOrder order)
        {
            col = column;
            innerOrder = order;
        }

        #endregion

        #region Methods

        /// <summary>
        /// Compare tow objects
        /// </summary>
        /// <param name="x">object 1</param>
        /// <param name="y">object 2</param>
        /// <returns></returns>
        public int Compare(object x, object y)
        {
            return Compare((ListViewItem)x, (ListViewItem)y);
        }

        /// <summary>
        /// Compare tow listview items
        /// </summary>
        /// <param name="x">Listview item 1</param>
        /// <param name="y">Listview item 2</param>
        /// <returns></returns>
        public int Compare(ListViewItem x, ListViewItem y)
        {
            if (innerOrder == SortOrder.Ascending)
            {
                return String.Compare(x.SubItems[col].Text, y.SubItems[col].Text);
            }
            else
            {
                return String.Compare(y.SubItems[col].Text, x.SubItems[col].Text);
            }
        }
       
        #endregion
    }
}